/*-----------------------------------------------------------------------*/
/* Joyous Calendar */
/*-----------------------------------------------------------------------*/

.joy-view-choices {
    float:                 right;
    text-align:            right;
}

.joy-view-choices__link {
}

.joy-cal {
    table-layout:          fixed; 
    border-spacing:        0;
    border-collapse:       separate;
    width:                 100%;
    border-collapse:       collapse;
}

.joy-cal__head {
}

.joy-cal__heading {
    border:                1px solid #aaa;
}

.joy-cal__period-heading {
    display:               inline-block;
    padding:               0 4ex;
}

a.joy-cal__period-ctrl {
}

.joy-cal__month-name {
    display:               inline-block;
    width:                 10ex;
}

.joy-cal__week-name {
    display:               inline-block;
    width:                 10ex;
}

.joy-cal__year-number {
    display:               inline-block;
    width:                 6ex;
}

.joy-cal__weekday {
    border:                1px solid #aaa;
}

.joy-cal__body {
}

.joy-cal__no-day,
.joy-cal__day {
    width:                 14.285%;
    position:              relative;
    overflow:              hidden;
    vertical-align:        top;
    border:                1px solid #aaa;
}

.joy-cal__day--yesterday {
}

.joy-cal__day--lastweek {
}

.joy-cal__day--today {
}

.joy-cal__heading:last-child,
.joy-cal__weekday:last-child,
.joy-cal__day:last-child {
}

.joy-cal__week:last-child .joy-cal__day {
}

.joy-cal__week:last-child .joy-cal__day:first-child {
}

.joy-cal__week:last-child .joy-cal__day:last-child {
}

.joy-cal__day-title {
    min-height:            25px;
    margin:                0;
    padding:               2px 0px 0px 0px;
}

.joy-cal__holiday-name {
    margin:                0;
    padding:               2px 0 1px 30px;
}

.joy-cal--weekly .joy-cal__holiday-name {
    padding-left:          0px;
}

.joy-cal__date {
    padding:               0px 0 1px 2px;
    float:                 left;
}

.joy-cal--weekly .joy-cal__date {
    float:                 none;
}

.joy-cal__weekday--sun,
.joy-cal__date--holiday,
.joy-cal__date--sun {
}

.joy-days-events {
    clear:                 both;
    min-width:             8ex;
}
.joy-cal--weekly .joy-days-events {
    min-height:            64ex;
}
.joy-days-events__event {
    display:               block;
    line-height:           16px;
    padding-left:          3ex;
    text-indent:          -3ex;
}
a.joy-days-events__event {

}

a.joy-days-events__event:active {
}

.joy-days-events__event-time {
    padding:               0 1ex 0 0;
}

.joy-days-events__event-title {
}

.joy-days-events__event-text {
}

a.joy-cal__read-more {
    position:              absolute;
    bottom:                0;
    right:                 0;
    text-align:            right;
    margin:                0px;
    padding:               5px;
}

.joy-overlay {
    position:              fixed;
    top:                   0;
    left:                  0;
    width:                 100%;
    height:                100%;
    z-index:               2;
    display:               none;
}

.joy-popup__outer {
    position:              absolute;
    top:                   0;
    left:                  0;
    width:                 100%;
    z-index:               3;
    display:               none;
}

.joy-popup__content {
    position:              relative;
    padding:               20px;
    max-width:             500px;
    width:                 90%;
    height:                auto;
    margin:                10px auto 10px auto;
    overflow:              auto;
    background:            white;
    border:                1px solid #aaa;
}

a.joy-popup__close {
    position:              absolute;
    top:                   5px;
    right:                 20px;
}

.joy-export {
    padding-top:           22px;
}

/*-----------------------------------------------------------------------*/
/* Joyous MiniCalendar */
/*-----------------------------------------------------------------------*/

.joy-minical {
    border-collapse:       collapse;
}

.joy-minical__heading {
    padding:               5px;
    font-weight:           bold;
    text-align:            center;
}

.joy-minical__month-heading {
    display:               inline-block;
    padding:               0 0 0 4ex;
    text-align:            left;
}

.joy-minical__month-name {
    display:               inline-block;
    width:                 10ex;
}

.joy-minical__year-number {
    display:               inline-block;
    width:                 6ex;
}

.joy-minical__month-nav {
    display:               inline-block;
    padding:               0 4ex 0 0;
    float:                 right;
}

.joy-minical__weekday {
    padding:               5px;
    font-weight:           bold;
}

.joy-minical__weekday--sun, {
}

.joy-minical__day {
}

.joy-minical__no-day {
}

.joy-minical__day-title {
    margin:                0;
    padding:               8px 8px 9px 8px;
    float:                 right;
}

.joy-minical__day-title--today {
}

.joy-minical__date--event {
    font-weight:           bold;
}

a.joy-minical__date--event-link {
    font-weight:           bold;
}

a.joy-minical__date--event-link:hover {
}

.joy-minical__date--holiday,
.joy-minical__date--sun {

}

a.joy-minical__date--holiday,
a.joy-minical__date--sun {
}

a.joy-minical__date--holiday:hover,
a.joy-minical__date--sun:hover {
}

/*-----------------------------------------------------------------------*/
/* Joyous Events */
/*-----------------------------------------------------------------------*/

.joy-ev-status {
    float:                 right;
}

.joy-ev-item {
}

.joy-field {
    display:               block;
}

a.joy-img-link {
    background-color:      transparent;
    text-decoration:       none;
    outline:               0;
}

.joy-img-link__img {
}

.joy-img__img {
}

.joy-ev-who {
}

.joy-ev-when,
.joy-ev-next-on,
.joy-ev-where {
}

.joy-ev-from-when {
}

.joy-ev-extra {
}

.joy-ev-related {
}

.joy-ev-related__item {
}

.joy-ev-related__link {
}

.joy-ev-related__overrides {
}

.joy-ev-details {
}

/*-----------------------------------------------------------------------*/
/* Events This Week*/
/*-----------------------------------------------------------------------*/

.joy-this-week {
    float:                 right;
    margin:                30px 50px 10px 20px;
    overflow:              hidden;
}

.joy-this-week__title {
    padding:               5px;
}

.joy-this-week__day {
    margin:                0;
    padding:               4px 10px 4px 20px;
}

.joy-this-week__day--in-past {
}

.joy-this-week__day--today {
    padding:               8px 10px 6px 20px;
}

.joy-this-week__day-title {
    display:               inline-block;
    vertical-align:        top;
    width:                 110px;
}

.joy-this-week__weekday {
    margin:                0;
    padding:               0;
}

.joy-this-week__date {
}

.joy-this-week__days-events {
    display:               inline-block;
    width:                 200px;
    padding:               3px 0 0 0;
}

.joy-this-week__holiday-name {
    display:               block;
    font-size:             12px;
    line-height:           16px;
}

.joy-this-week__cal {
    display:               block;
    text-align:            center;
    padding:               2px 1px 4px 1px;
}

/*-----------------------------------------------------------------------*/
/* Pagination */
/*-----------------------------------------------------------------------*/

.joy-pg {
  display:                 inline-block;
  padding-left:            0;
}

.joy-pg__page {
    display:               inline;
}

.joy-pg__ctrl,
.joy-pg__num {
      position:            relative;
      float:               left;
      padding:             10px;
      text-decoration:     none;
      border:              1px solid #aaa;
      margin-left:         -1px;
}

.joy-pg__ctrl--disabled {
}

.joy-pg__num--active {
}
