{% load static wagtailuserbar %}

<!DOCTYPE html>
<!--[if lt IE 7]>      <html class="no-js lt-ie9 lt-ie8 lt-ie7"> <![endif]-->
<!--[if IE 7]>         <html class="no-js lt-ie9 lt-ie8"> <![endif]-->
<!--[if IE 8]>         <html class="no-js lt-ie9"> <![endif]-->
<!--[if gt IE 8]><!--> <html class="no-js"> <!--<![endif]-->
    <head>
        <meta charset="utf-8" />
        <meta http-equiv="X-UA-Compatible" content="IE=edge" />
        <title>
            {% block title %}
                {% if self.seo_title %}{{ self.seo_title }}{% else %}{{ self.title }}{% endif %}
            {% endblock %}
            {% block title_suffix %}
                {% with self.get_site.site_name as site_name %}
                    {% if site_name %}- {{ site_name }}{% endif %}
                {% endwith %}
            {% endblock %}
        </title>
        <meta name="description" content="" />
        <meta name="viewport" content="width=device-width, initial-scale=1" />
        {% block global_css %}
        {# Global stylesheets #}
        {% endblock %}
        {% block extra_css %}
        {# Override this in templates to add extra stylesheets #}
        {% endblock %}
    </head>

    <body class="{% block body_class %}{% endblock %}">
        {% wagtailuserbar %}
        <div id="wrapper">
          {% block background %}
          {% endblock background %}

          {% block header %}
          <header >
            {% block banner %}
            <div class="banner clearfix">
              <h1>Joyous Tests</h1>
              <p>
                A cut-down boilerplate base.html for use with the unittests.
              </p>
            </div>
            {% endblock banner %}
          </header>
          {% endblock header %}

          {% block main %}
          <main id="main">
            {% block content %}
            {% endblock %}
          </main>
          {% endblock main %}
        </div> <!-- wrapper -->

        {% block footer %}
        <footer>
          <div class="back-to-top">
            <a href="#wrapper">Back to Top</a>
          </div>
          <div class="bottom clearfix">
            <div class="bottom-left">
              <div class="login">
               <a href="/admin/login/">Login</a>
              </div>
            </div>
          </div>
        </footer>
        {% endblock footer %}

        {# Global javascript #}
        <script src="{% static 'js/vendor/jquery-1.12.4.min.js' %}"></script>
        <script src="{% static 'js/vendor/ie10-viewport-bug-workaround.js' %}"></script>

        {% block extra_js %}
        {# Override this in templates to add extra javascript #}
        {% endblock %}
    </body>
</html>
