# Generated by Django 2.0.7 on 2018-07-15 05:56

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('joyous', '0006_generalcalendarpage_specificcalendarpage'),
    ]

    operations = [
        migrations.AddField(
            model_name='multidayeventpage',
            name='uid',
            field=models.CharField(db_index=True, default=uuid.uuid4, editable=False, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='postponementpage',
            name='uid',
            field=models.CharField(db_index=True, default=uuid.uuid4, editable=False, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='recurringeventpage',
            name='uid',
            field=models.CharField(db_index=True, default=uuid.uuid4, editable=False, max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='simpleeventpage',
            name='uid',
            field=models.CharField(db_index=True, default=uuid.uuid4, editable=False, max_length=255, null=True),
        ),
    ]
